@@ -16,15 +16,16 @@
 //
 
 #include "QtCore/qxmlstream.h"
-#include "QtCore/qhash.h"
 #include "QtCore/qstack.h"
 #include <QtCore/QLoggingCategory>
 #include "qsvgstyle_p.h"
 #if QT_CONFIG(cssparser)
-#include "private/qcssparser_p.h"
+#include <QtSvg/private/qsvgcsshandler_p.h>
+#include <QtSvg/private/qsvgcssproperties_p.h>
 #endif
 #include "qsvggraphics_p.h"
 #include "qtsvgglobal_p.h"
+#include "qsvgutils_p.h"
 
 QT_BEGIN_NAMESPACE
 
@@ -32,36 +33,16 @@ class QSvgNode;
 class QSvgTinyDocument;
 class QSvgHandler;
 class QColor;
-class QSvgStyleSelector;
 
-#ifndef QT_NO_CSSPARSER
-
-struct QSvgCssAttribute
-{
-    QString name;
-    QString value;
-};
-
-#endif
-
-class Q_SVG_PRIVATE_EXPORT QSvgHandler
+class Q_SVG_EXPORT QSvgHandler
 {
 public:
-    enum LengthType {
-        LT_PERCENT,
-        LT_PX,
-        LT_PC,
-        LT_PT,
-        LT_MM,
-        LT_CM,
-        LT_IN,
-        LT_OTHER
-    };
-
-public:
-    QSvgHandler(QIODevice *device, QtSvg::Options options = {});
-    QSvgHandler(const QByteArray &data, QtSvg::Options options = {});
-    QSvgHandler(QXmlStreamReader *const data, QtSvg::Options options = {});
+    QSvgHandler(QIODevice *device, QtSvg::Options options = {},
+                QtSvg::AnimatorType type = QtSvg::AnimatorType::Automatic);
+    QSvgHandler(const QByteArray &data, QtSvg::Options options = {},
+                QtSvg::AnimatorType type = QtSvg::AnimatorType::Automatic);
+    QSvgHandler(QXmlStreamReader *const data, QtSvg::Options options = {},
+                QtSvg::AnimatorType type = QtSvg::AnimatorType::Automatic);
     ~QSvgHandler();
 
     QIODevice *device() const;
@@ -74,8 +55,8 @@ public:
     inline QString errorString() const { return xml->errorString(); }
     inline int lineNumber() const { return xml->lineNumber(); }
 
-    void setDefaultCoordinateSystem(LengthType type);
-    LengthType defaultCoordinateSystem() const;
+    void setDefaultCoordinateSystem(QSvgUtils::LengthType type);
+    QSvgUtils::LengthType defaultCoordinateSystem() const;
 
     void pushColor(const QColor &color);
     void pushColorCopy();
@@ -86,27 +67,24 @@ public:
     void setInStyle(bool b);
     bool inStyle() const;
 
-    QSvgStyleSelector *selector() const;
+    QSvgCssHandler &cssHandler();
 #endif
 
     void setAnimPeriod(int start, int end);
     int animationDuration() const;
 
-#ifndef QT_NO_CSSPARSER
-    void parseCSStoXMLAttrs(const QString &css, QList<QSvgCssAttribute> *attributes);
-#endif
-
     inline QPen defaultPen() const
     { return m_defaultPen; }
 
     QtSvg::Options options() const;
+    QtSvg::AnimatorType animatorType() const;
     bool trustedSourceMode() const;
 
 public:
-    bool startElement(const QString &localName, const QXmlStreamAttributes &attributes);
-    bool endElement(QStringView localName);
-    bool characters(QStringView str);
-    bool processingInstruction(const QString &target, const QString &data);
+    bool startElement(const QStringView localName, const QXmlStreamAttributes &attributes);
+    bool endElement(const QStringView localName);
+    bool characters(const QStringView str);
+    bool processingInstruction(const QStringView target, const QStringView data);
 
 private:
     void init();
@@ -114,14 +92,17 @@ private:
     QSvgTinyDocument *m_doc;
     QStack<QSvgNode *> m_nodes;
     // TODO: This is only needed during parsing, so it unnecessarily takes up space after that.
-    // Temporary container for <use> nodes which haven't been resolved yet.
-    QList<QSvgUse *> m_toBeResolved;
+    // Temporary container for :
+    // - <use> nodes which haven't been resolved yet.
+    // - <filter> nodes to be checked for unsupported filter primitives.
+    QList<QSvgNode *> m_toBeResolved;
 
     enum CurrentNode
     {
         Unknown,
         Graphics,
-        Style
+        Style,
+        Doc
     };
     QStack<CurrentNode> m_skipNodes;
 
@@ -133,7 +114,7 @@ private:
 
     QSvgRefCounter<QSvgStyleProperty> m_style;
 
-    LengthType m_defaultCoords;
+    QSvgUtils::LengthType m_defaultCoords;
 
     QStack<QColor> m_colorStack;
     QStack<int>    m_colorTagCount;
@@ -143,8 +124,7 @@ private:
     QXmlStreamReader *const xml;
 #ifndef QT_NO_CSSPARSER
     bool m_inStyle;
-    QSvgStyleSelector *m_selector;
-    QCss::Parser m_cssParser;
+    QSvgCssHandler m_cssHandler;
 #endif
     void parse();
     void resolvePaintServers(QSvgNode *node, int nestedDepth = 0);
@@ -158,6 +138,7 @@ private:
     const bool m_ownsReader;
 
     const QtSvg::Options m_options;
+    const QtSvg::AnimatorType m_animatorType;
 };
 
 Q_DECLARE_LOGGING_CATEGORY(lcSvgHandler)
